@@ -82,12 +82,15 @@
 #define KEY_LOGO                    KEY_GREETER_DIR "/logo_icon_name"
 #define KEY_DISABLE_USER_LIST       "/apps/gdm/simple-greeter/disable_user_list"
 
+#define LSB_RELEASE_COMMAND "lsb_release -d"
+
 #define GDM_GREETER_LOGIN_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_GREETER_LOGIN_WINDOW, GdmGreeterLoginWindowPrivate))
 
 enum {
         MODE_UNDEFINED = 0,
+        MODE_TIMED_LOGIN,
         MODE_SELECTION,
-        MODE_AUTHENTICATION
+        MODE_AUTHENTICATION,
 };
 
 enum {
@@ -101,6 +104,8 @@ struct GdmGreeterLoginWindowPrivate
         GtkBuilder      *builder;
         GtkWidget       *user_chooser;
         GtkWidget       *auth_banner_label;
+        GtkWidget       *current_button;
+
         guint            display_is_local : 1;
         guint            is_interactive : 1;
         guint            user_chooser_loaded : 1;
@@ -109,10 +114,11 @@ struct GdmGreeterLoginWindowPrivate
         gboolean         banner_message_enabled;
         guint            gconf_cnxn;
 
+        guint            last_mode;
         guint            dialog_mode;
 
         gboolean         user_list_disabled;
-        gboolean         show_cancel_button;
+        guint            num_queries;
 
         gboolean         timed_login_already_enabled;
         gboolean         timed_login_enabled;
@@ -333,19 +339,6 @@ show_widget (GdmGreeterLoginWindow *login_window,
         }
 }
 
-static void
-sensitize_widget (GdmGreeterLoginWindow *login_window,
-                  const char            *name,
-                  gboolean               sense)
-{
-        GtkWidget *widget;
-
-        widget = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, name));
-        if (widget != NULL) {
-                gtk_widget_set_sensitive (widget, sense);
-        }
-}
-
 static void
 on_login_button_clicked_answer_query (GtkButton             *button,
                                       GdmGreeterLoginWindow *login_window)
@@ -378,15 +371,48 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
                         int                    mode)
 {
         GtkWidget *button;
+        GtkWidget *login_button;
+        GtkWidget *unlock_button;
+        char      *item;
+        gboolean   in_use;
 
-        button = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "log-in-button"));
+        in_use = FALSE;
+        item = gdm_chooser_widget_get_active_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        if (item != NULL) {
+                gboolean res;
+
+                res = gdm_chooser_widget_lookup_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                      item,
+                                                      NULL, /* image */
+                                                      NULL, /* name */
+                                                      NULL, /* comment */
+                                                      NULL, /* priority */
+                                                      &in_use,
+                                                      NULL); /* is separate */
+        }
+
+        if (login_window->priv->current_button != NULL) {
+                /* disconnect any signals */
+                if (login_window->priv->login_button_handler_id > 0) {
+                        g_signal_handler_disconnect (login_window->priv->current_button,
+                                                     login_window->priv->login_button_handler_id);
+                        login_window->priv->login_button_handler_id = 0;
+                }
+        }
+
+        unlock_button = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "unlock-button"));
+        login_button = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "log-in-button"));
+
+        if (in_use) {
+                gtk_widget_hide (login_button);
+                button = unlock_button;
+        } else {
+                gtk_widget_hide (unlock_button);
+                button = login_button;
+        }
         gtk_widget_grab_default (button);
 
-        /* disconnect any signals */
-        if (login_window->priv->login_button_handler_id > 0) {
-                g_signal_handler_disconnect (button, login_window->priv->login_button_handler_id);
-                login_window->priv->login_button_handler_id = 0;
-       }
+        login_window->priv->current_button = button;
 
         switch (mode) {
         case LOGIN_BUTTON_HIDDEN:
@@ -406,22 +432,55 @@ set_log_in_button_mode (GdmGreeterLoginWindow *login_window,
         }
 }
 
-static void
-adjust_other_login_visibility(GdmGreeterLoginWindow *login_window)
+static gboolean
+user_chooser_has_no_user (GdmGreeterLoginWindow *login_window)
 {
-        if (! login_window->priv->user_chooser_loaded) {
-                return;
+        guint num_items;
+
+        num_items = gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        g_debug ("GdmGreeterLoginWindow: loaded=%d num_items=%d",
+                 login_window->priv->user_chooser_loaded,
+                 num_items);
+        return (login_window->priv->user_chooser_loaded && num_items == 0);
+}
+
+static void
+maybe_show_cancel_button (GdmGreeterLoginWindow *login_window)
+{
+        gboolean show;
+
+        show = FALSE;
+
+        /* only show the cancel button if there is something to go
+           back to */
+
+        switch (login_window->priv->dialog_mode) {
+        case MODE_SELECTION:
+                /* should never have anything to return to from here */
+                show = FALSE;
+                break;
+        case MODE_TIMED_LOGIN:
+                /* should always have something to return to from here */
+                show = TRUE;
+                break;
+        case MODE_AUTHENTICATION:
+                if (login_window->priv->num_queries > 1) {
+                        /* if we are inside a pam conversation past
+                           the first step */
+                        show = TRUE;
+                } else {
+                        if (login_window->priv->user_list_disabled || user_chooser_has_no_user (login_window)) {
+                                show = FALSE;
+                        } else {
+                                show = TRUE;
+                        }
+                }
+                break;
+        default:
+                g_assert_not_reached ();
         }
 
-        if (login_window->priv->dialog_mode != MODE_SELECTION) {
-                return;
-        }
-
-        if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
-                set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
-        } else {
-                set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
-        }
+        show_widget (login_window, "cancel-button", show);
 }
 
 static void
@@ -437,57 +496,35 @@ switch_mode (GdmGreeterLoginWindow *login_window,
         /* we want to run this even if we're supposed to
            be in the mode already so that we reset everything
            to a known state */
-        login_window->priv->dialog_mode = number;
+        if (login_window->priv->dialog_mode != number) {
+                login_window->priv->last_mode = login_window->priv->dialog_mode;
+                login_window->priv->dialog_mode = number;
+        }
 
         default_name = NULL;
 
         switch (number) {
         case MODE_SELECTION:
                 set_log_in_button_mode (login_window, LOGIN_BUTTON_HIDDEN);
-
-                show_widget (login_window, "cancel-button", FALSE);
-
-                show_widget (login_window, "auth-input-box", FALSE);
-
-                sensitize_widget (login_window, "disconnect-button", FALSE);
-
-                /*
-                 * Although the cancel button is not shown in the selection
-                 * dialog, the show_cancel_button flag manages when the
-                 * cancel button is shown on subsequent PAM queries.  Set the
-                 * flag to true initially since the button should appear on
-                 * the first query, since the button takes the user back to
-                 * selection mode.
-                 */
-                login_window->priv->show_cancel_button = TRUE;
+                break;
+        case MODE_TIMED_LOGIN:
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
                 break;
         case MODE_AUTHENTICATION:
-                show_widget (login_window, "disconnect-button", FALSE);
-                default_name = "log-in-button";
+                set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
                 break;
         default:
                 g_assert_not_reached ();
         }
 
-        box = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder,
-                          "buttonbox"));
-        gtk_button_box_set_layout (GTK_BUTTON_BOX (box),
-                                   (number == MODE_SELECTION) ?
-                                    GTK_BUTTONBOX_SPREAD :
-                                    GTK_BUTTONBOX_END );
-
-        if (default_name != NULL) {
-                GtkWidget *widget;
-
-                widget = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, default_name));
-                gtk_widget_grab_default (widget);
-        }
+        show_widget (login_window, "auth-input-box", FALSE);
+        maybe_show_cancel_button (login_window);
 
         /*
          * The rest of this function sets up the user list, so just return if
          * the user list is disabled.
          */
-        if (login_window->priv->user_list_disabled) {
+        if (login_window->priv->user_list_disabled && number != MODE_TIMED_LOGIN) {
                 return;
         }
 
@@ -512,70 +549,134 @@ switch_mode (GdmGreeterLoginWindow *login_window,
 }
 
 static void
-delete_entry_text (GtkWidget *entry)
+choose_user (GdmGreeterLoginWindow *login_window,
+             const char            *user_name)
 {
-        const char *typed_text;
-        char       *null_text;
+        guint mode;
 
-        /* try to scrub out any secret info */
-        typed_text = gtk_entry_get_text (GTK_ENTRY (entry));
-        null_text = g_strnfill (strlen (typed_text) + 1, '\b');
-        gtk_entry_set_text (GTK_ENTRY (entry), null_text);
-        gtk_entry_set_text (GTK_ENTRY (entry), "");
+        g_assert (user_name != NULL);
+
+        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                       0, user_name);
+
+        mode = MODE_AUTHENTICATION;
+        if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
+                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
+                /* FIXME: handle guest account stuff */
+        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
+                g_signal_emit (login_window, signals[BEGIN_AUTO_LOGIN], 0,
+                               login_window->priv->timed_login_username);
+
+                login_window->priv->timed_login_enabled = TRUE;
+                restart_timed_login_timeout (login_window);
+
+                /* just wait for the user to select language and stuff */
+                mode = MODE_TIMED_LOGIN;
+                set_message (login_window, _("Select language and click Log In"));
+        } else {
+                g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
+        }
+
+        switch_mode (login_window, mode);
 }
 
 static void
-reset_dialog (GdmGreeterLoginWindow *login_window)
+retry_login (GdmGreeterLoginWindow *login_window)
+{
+        GtkWidget  *entry;
+        char       *user_name;
+
+        user_name = gdm_user_chooser_widget_get_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        if (user_name == NULL) {
+                return;
+        }
+
+        g_debug ("GdmGreeterLoginWindow: Retrying login for %s", user_name);
+
+        entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
+        gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1);
+
+        choose_user (login_window, user_name);
+
+        g_free (user_name);
+}
+
+static gboolean
+can_jump_to_authenticate (GdmGreeterLoginWindow *login_window)
+{
+        gboolean res;
+
+        if (!login_window->priv->user_chooser_loaded) {
+                res = FALSE;
+        } else if (login_window->priv->user_list_disabled) {
+                res = (login_window->priv->timed_login_username == NULL);
+        } else {
+                res = user_chooser_has_no_user (login_window);
+        }
+
+        return res;
+}
+
+static void
+reset_dialog (GdmGreeterLoginWindow *login_window,
+              guint                  dialog_mode)
 {
         GtkWidget  *entry;
         GtkWidget  *label;
+        guint       mode;
 
-        g_debug ("GdmGreeterLoginWindow: Resetting dialog");
+        g_debug ("GdmGreeterLoginWindow: Resetting dialog to mode %u", dialog_mode);
         set_busy (login_window);
         set_sensitive (login_window, FALSE);
 
-        login_window->priv->show_cancel_button = FALSE;
+        login_window->priv->num_queries = 0;
 
-        if (login_window->priv->timed_login_enabled) {
-                gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
-                                                   GDM_USER_CHOOSER_USER_AUTO, 0);
-                remove_timed_login_timeout (login_window);
-                login_window->priv->timed_login_enabled = FALSE;
-        }
-        _gdm_greeter_login_window_set_interactive (login_window, FALSE);
+        if (dialog_mode == MODE_SELECTION) {
+                if (login_window->priv->timed_login_enabled) {
+                        gdm_chooser_widget_set_item_timer (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser),
+                                                           GDM_USER_CHOOSER_USER_AUTO, 0);
+                        remove_timed_login_timeout (login_window);
+                        login_window->priv->timed_login_enabled = FALSE;
+                }
+                _gdm_greeter_login_window_set_interactive (login_window, FALSE);
 
-        g_signal_handlers_block_by_func (G_OBJECT (login_window->priv->user_chooser),
-                                         G_CALLBACK (on_user_unchosen), login_window);
-        gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
-        g_signal_handlers_unblock_by_func (G_OBJECT (login_window->priv->user_chooser),
-                                           G_CALLBACK (on_user_unchosen), login_window);
+                g_signal_handlers_block_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                                 G_CALLBACK (on_user_unchosen), login_window);
+                gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), NULL);
+                g_signal_handlers_unblock_by_func (G_OBJECT (login_window->priv->user_chooser),
+                                                   G_CALLBACK (on_user_unchosen), login_window);
 
-        if (login_window->priv->start_session_handler_id > 0) {
-                g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
-                login_window->priv->start_session_handler_id = 0;
+                if (login_window->priv->start_session_handler_id > 0) {
+                        g_signal_handler_disconnect (login_window, login_window->priv->start_session_handler_id);
+                        login_window->priv->start_session_handler_id = 0;
+                }
+
+                set_message (login_window, "");
         }
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
 
-        delete_entry_text (entry);
+        gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1);
 
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
-        set_message (login_window, "");
 
         label = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-label"));
         gtk_label_set_text (GTK_LABEL (label), "");
 
-        if (login_window->priv->user_list_disabled) {
+        if (can_jump_to_authenticate (login_window)) {
+                /* If we don't have a user list jump straight to authenticate */
+                g_debug ("GdmGreeterLoginWindow: jumping straight to authenticate");
                 switch_mode (login_window, MODE_AUTHENTICATION);
         } else {
-                switch_mode (login_window, MODE_SELECTION);
+                switch_mode (login_window, dialog_mode);
         }
 
         set_sensitive (login_window, TRUE);
         set_ready (login_window);
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
         update_banner_message (login_window);
-        adjust_other_login_visibility (login_window);
+
         if (gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) >= 1) {
                 gdm_chooser_widget_propagate_pending_key_events (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
         }
@@ -596,29 +697,47 @@ gdm_greeter_login_window_ready (GdmGreeterLoginWindow *login_window)
 {
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        reset_dialog (login_window);
-
         set_sensitive (GDM_GREETER_LOGIN_WINDOW (login_window), TRUE);
         set_ready (GDM_GREETER_LOGIN_WINDOW (login_window));
         set_focus (GDM_GREETER_LOGIN_WINDOW (login_window));
 
-        /* If the user list is disabled, then start the PAM conversation */
-        if (login_window->priv->user_list_disabled) {
-                g_debug ("Starting PAM conversation since user list disabled");
-                g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
-                               0, GDM_USER_CHOOSER_USER_OTHER);
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+        /* If we are retrying a previously selected user */
+        if (!login_window->priv->user_list_disabled &&
+            login_window->priv->dialog_mode == MODE_AUTHENTICATION) {
+                retry_login (login_window);
+        } else {
+                /* If the user list is disabled, then start the PAM conversation */
+                if (can_jump_to_authenticate (login_window)) {
+                        g_debug ("Starting PAM conversation since user list disabled");
+                        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                                       0, GDM_USER_CHOOSER_USER_OTHER);
+                        g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+                }
         }
 
         return TRUE;
 }
 
+gboolean
+gdm_greeter_login_window_authentication_failed (GdmGreeterLoginWindow *login_window)
+{
+        g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
+
+        g_debug ("GdmGreeterLoginWindow: got authentication failed");
+
+        /* FIXME: shake? */
+        reset_dialog (login_window, MODE_AUTHENTICATION);
+
+        return TRUE;
+}
+
 gboolean
 gdm_greeter_login_window_reset (GdmGreeterLoginWindow *login_window)
 {
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        reset_dialog (GDM_GREETER_LOGIN_WINDOW (login_window));
+        g_debug ("GdmGreeterLoginWindow: got reset");
+        reset_dialog (login_window, MODE_SELECTION);
 
         return TRUE;
 }
@@ -651,25 +770,22 @@ gdm_greeter_login_window_problem (GdmGreeterLoginWindow *login_window,
 }
 
 static void
-handle_request_timed_login (GdmGreeterLoginWindow *login_window)
+request_timed_login (GdmGreeterLoginWindow *login_window)
 {
+        g_debug ("GdmGreeterLoginWindow: requesting timed login");
+
+        gtk_widget_show (login_window->priv->user_chooser);
+
         if (login_window->priv->dialog_mode != MODE_SELECTION) {
-                reset_dialog (login_window);
+                reset_dialog (login_window, MODE_SELECTION);
         }
-        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
         if (!login_window->priv->timed_login_already_enabled) {
                 gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser),
                                                               GDM_USER_CHOOSER_USER_AUTO);
         }
-}
 
-static void
-on_request_timed_login_after_users_loaded (GdmUserChooserWidget  *user_chooser,
-                                           GdmGreeterLoginWindow *login_window)
-{
-        g_debug ("Users now loaded, handling timed login request");
-        handle_request_timed_login (login_window);
+        login_window->priv->timed_login_already_enabled = TRUE;
 }
 
 void
@@ -681,24 +797,20 @@ gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_windo
 
         g_debug ("GdmGreeterLoginWindow: requested automatic login for user '%s' in %d seconds", username, delay);
 
-        if (login_window->priv->timed_login_username != NULL) {
-                login_window->priv->timed_login_already_enabled = TRUE;
-                g_free (login_window->priv->timed_login_username);
-        } else {
-                login_window->priv->timed_login_already_enabled = FALSE;
-        }
+        g_free (login_window->priv->timed_login_username);
         login_window->priv->timed_login_username = g_strdup (username);
         login_window->priv->timed_login_delay = delay;
 
+        /* add the auto user right away so we won't trigger a mode
+           switch to authenticate when the user list is disabled */
+        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+
+        /* if the users aren't loaded then we'll handle it in when they are */
         if (login_window->priv->user_chooser_loaded) {
                 g_debug ("Handling timed login request since users are already loaded.");
-                handle_request_timed_login (login_window);
+                request_timed_login (login_window);
         } else {
                 g_debug ("Waiting to handle timed login request until users are loaded.");
-                g_signal_connect (login_window->priv->user_chooser,
-                                  "loaded",
-                                  G_CALLBACK (on_request_timed_login_after_users_loaded),
-                                  login_window);
         }
 }
 
@@ -739,21 +851,6 @@ gdm_greeter_login_window_start_session_when_ready (GdmGreeterLoginWindow *login_
         }
 }
 
-static void
-_show_cancel_button (GdmGreeterLoginWindow *login_window)
-{
-        /*
-         * If show_cancel_button is false, this is because the face browser
-         * is disabled and the first query is showing.  It does not make sense
-         * to display the cancel button when showing the initial query and
-         * the face browser is disabled.  Set the flag to true so that the
-         * button is shown on following queries.
-         */
-        show_widget (login_window, "cancel-button",
-                     login_window->priv->show_cancel_button);
-        login_window->priv->show_cancel_button = TRUE;
-}
-
 gboolean
 gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
                                      const char            *text)
@@ -763,12 +860,13 @@ gdm_greeter_login_window_info_query (GdmGreeterLoginWindow *login_window,
 
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        _show_cancel_button (login_window);
+        login_window->priv->num_queries++;
+        maybe_show_cancel_button (login_window);
 
         g_debug ("GdmGreeterLoginWindow: info query: %s", text);
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
-        delete_entry_text (entry);
+        gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1);
         gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
         set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
@@ -794,10 +892,11 @@ gdm_greeter_login_window_secret_info_query (GdmGreeterLoginWindow *login_window,
 
         g_return_val_if_fail (GDM_IS_GREETER_LOGIN_WINDOW (login_window), FALSE);
 
-        _show_cancel_button (login_window);
+        login_window->priv->num_queries++;
+        maybe_show_cancel_button (login_window);
 
         entry = GTK_WIDGET (gtk_builder_get_object (GDM_GREETER_LOGIN_WINDOW (login_window)->priv->builder, "auth-prompt-entry"));
-        delete_entry_text (entry);
+        gtk_editable_delete_text (GTK_EDITABLE (entry), 0, -1);
         gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
         set_log_in_button_mode (login_window, LOGIN_BUTTON_ANSWER_QUERY);
 
@@ -888,8 +987,8 @@ cancel_button_clicked (GtkButton             *button,
 static void
 on_user_chooser_visibility_changed (GdmGreeterLoginWindow *login_window)
 {
+        g_debug ("GdmGreeterLoginWindow: Chooser visibility changed");
         update_banner_message (login_window);
-        adjust_other_login_visibility (login_window);
 }
 
 static void
@@ -898,10 +997,27 @@ on_users_loaded (GdmUserChooserWidget  *user_chooser,
 {
         g_debug ("GdmGreeterLoginWindow: users loaded");
         login_window->priv->user_chooser_loaded = TRUE;
-        update_banner_message (login_window);
-        adjust_other_login_visibility (login_window);
 
-        gdm_chooser_widget_activate_if_one_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        update_banner_message (login_window);
+
+        if (!login_window->priv->user_list_disabled) {
+                gtk_widget_show (login_window->priv->user_chooser);
+        }
+
+        if (login_window->priv->timed_login_username != NULL
+            && !login_window->priv->timed_login_already_enabled) {
+                request_timed_login (login_window);
+        } else if (can_jump_to_authenticate (login_window)) {
+                /* jump straight to authenticate */
+                g_debug ("GdmGreeterLoginWindow: jumping straight to authenticate");
+
+                switch_mode (login_window, MODE_AUTHENTICATION);
+
+                g_debug ("Starting PAM conversation since no local users");
+                g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
+                               0, GDM_USER_CHOOSER_USER_OTHER);
+                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
+        }
 }
 
 static void
@@ -909,6 +1025,7 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 GdmGreeterLoginWindow *login_window)
 {
         char *user_name;
+        guint mode;
 
         user_name = gdm_user_chooser_widget_get_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser));
         g_debug ("GdmGreeterLoginWindow: user chosen '%s'", user_name);
@@ -917,29 +1034,7 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 return;
         }
 
-        g_signal_emit (G_OBJECT (login_window), signals[USER_SELECTED],
-                       0, user_name);
-
-        if (strcmp (user_name, GDM_USER_CHOOSER_USER_OTHER) == 0) {
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION], 0);
-        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_GUEST) == 0) {
-                /* FIXME: handle guest account stuff */
-        } else if (strcmp (user_name, GDM_USER_CHOOSER_USER_AUTO) == 0) {
-                g_signal_emit (login_window, signals[BEGIN_AUTO_LOGIN], 0,
-                               login_window->priv->timed_login_username);
-
-                login_window->priv->timed_login_enabled = TRUE;
-                restart_timed_login_timeout (login_window);
-
-                /* just wait for the user to select language and stuff */
-                set_log_in_button_mode (login_window, LOGIN_BUTTON_TIMED_LOGIN);
-                set_message (login_window, _("Select language and click Log In"));
-        } else {
-                g_signal_emit (login_window, signals[BEGIN_VERIFICATION_FOR_USER], 0, user_name);
-        }
-
-        switch_mode (login_window, MODE_AUTHENTICATION);
-
+        choose_user (login_window, user_name);
         g_free (user_name);
 }
 
@@ -1023,10 +1118,24 @@ static char *
 get_system_version (void)
 {
         char *version;
+        char *output;
         int i;
 
         version = NULL;
 
+        output = NULL;
+        if (g_spawn_command_line_sync (LSB_RELEASE_COMMAND, &output, NULL, NULL, NULL)) {
+                if (g_str_has_prefix (output, "Description:")) {
+                        version = g_strdup (output + strlen ("Description:"));
+                } else {
+                        version = g_strdup (output);
+                }
+                version = g_strstrip (version);
+                g_free (output);
+
+                goto out;
+        }
+
         for (i = 0; known_etc_info_files [i]; i++) {
                 char *path1;
                 char *path2;
@@ -1046,13 +1155,12 @@ get_system_version (void)
         }
 
         if (version == NULL) {
-                char *output;
                 output = NULL;
                 if (g_spawn_command_line_sync ("uname -sr", &output, NULL, NULL, NULL)) {
                         version = g_strchomp (output);
                 }
         }
-
+ out:
         return version;
 }
 
@@ -1170,10 +1278,6 @@ load_theme (GdmGreeterLoginWindow *login_window)
                                  G_CALLBACK (on_user_chooser_visibility_changed),
                                  login_window);
 
-        if (!login_window->priv->user_list_disabled) {
-                gtk_widget_show (login_window->priv->user_chooser);
-        }
-
         login_window->priv->auth_banner_label = GTK_WIDGET (gtk_builder_get_object (login_window->priv->builder, "auth-banner-label"));
         /*make_label_small_italic (login_window->priv->auth_banner_label);*/
 
@@ -1211,7 +1315,8 @@ gdm_greeter_login_window_key_press_event (GtkWidget   *widget,
         login_window = GDM_GREETER_LOGIN_WINDOW (widget);
 
         if (event->keyval == GDK_Escape) {
-                if (login_window->priv->dialog_mode == MODE_AUTHENTICATION) {
+                if (login_window->priv->dialog_mode == MODE_AUTHENTICATION
+                    || login_window->priv->dialog_mode == MODE_TIMED_LOGIN) {
                         do_cancel (GDM_GREETER_LOGIN_WINDOW (widget));
                 }
         }
@@ -1288,7 +1393,7 @@ update_banner_message (GdmGreeterLoginWindow *login_window)
         } else {
                 char *message = NULL;
                 error = NULL;
-                if (login_window->priv->user_chooser_loaded && gdm_chooser_widget_get_number_of_items (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser)) == 0) {
+                if (user_chooser_has_no_user (login_window)) {
                         message = gconf_client_get_string (login_window->priv->client, KEY_BANNER_MESSAGE_TEXT_NOCHOOSER, &error);
                         if (error != NULL) {
                                 g_debug("GdmGreeterLoginWindow: unable to get nochooser banner text: %s", error->message);
@@ -1502,7 +1607,6 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         GConfClient *client;
         GError      *error;
         gboolean     user_list_disable;
-        gboolean     timed_login_enable;
 
         gdm_profile_start (NULL);
 
@@ -1517,22 +1621,14 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
          * timed login is also not being used.
          */
         user_list_disable = gconf_client_get_bool (client,
-                KEY_DISABLE_USER_LIST, &error);
+                                                   KEY_DISABLE_USER_LIST,
+                                                   &error);
         if (error != NULL) {
                 g_debug ("GdmUserChooserWidget: unable to get disable-user-list configuration: %s", error->message);
                 g_error_free (error);
         }
 
-        gdm_settings_client_get_boolean (GDM_KEY_TIMED_LOGIN_ENABLE,
-                                         &timed_login_enable);
-
-        login_window->priv->user_list_disabled = FALSE;
-        if (user_list_disable && !timed_login_enable) {
-                g_debug ("Disabling user list");
-                login_window->priv->user_list_disabled = TRUE;
-        } else {
-                g_debug ("Enabling user list");
-        }
+        login_window->priv->user_list_disabled = user_list_disable;
 
         gtk_window_set_title (GTK_WINDOW (login_window), _("Login Window"));
         /*gtk_window_set_opacity (GTK_WINDOW (login_window), 0.85);*/
@@ -1543,7 +1639,6 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
-        gtk_window_set_type_hint (GTK_WINDOW (login_window), GDK_WINDOW_TYPE_HINT_DOCK);
         gtk_container_set_border_width (GTK_CONTAINER (login_window), 0);
 
         g_signal_connect (login_window,
